import { getMonthlyDay } from './utils';

function initPickerColumnsIndex() {
  const colsIndex = {};
  this.columns.forEach((col, indx) => {
    if (col.name) {
      colsIndex[col.name] = indx;
    }
  });
  return colsIndex;
}

class DateTime {
  constructor(config = {}) {
    this.config = {
      el: config.el || '',
      dateFormat:
        config.dateFormat || (config.dateTimePicker ? 'dd/mm/yyyy hh::mm A' : 'dd/mm/yyyy'),
      pickerFormat: config.dateTimePicker ? 'YYYY-MM-DD hh:mm' : 'YYYY-MM-DD',
      is24Hours: config.is24Hours || false,
      dateTimePicker: config.dateTimePicker || false,
      datePicker: config.datePicker || !config.dateTimePicker,
      startYear: config.startYear || 1900,
      endYear: config.endYear || 2200,
      defaultValue: config.defaultValue || null,
      minStep: config.minStep || 1,
      locale: config.locale || 'en-US',
    };
    this.isUpdating = false;
    this.initializing = false;
    this.dateTime = config.defaultValue || new Date();
    this.dayFormatter = new Intl.DateTimeFormat(this.config.locale, { day: '2-digit' });
    this.monthFormatter = new Intl.DateTimeFormat(this.config.locale, {
      month: config.dateTimePicker ? 'short' : 'long',
    });
    this.longMonthFormatter = new Intl.DateTimeFormat(this.config.locale, { month: 'long' });
    this.shortMonthFormatter = new Intl.DateTimeFormat(this.config.locale, { month: 'short' });
    this.yearFormatter = new Intl.DateTimeFormat(this.config.locale, { year: 'numeric' });
    this.createPickerColumns = this.createPickerColumns.bind(this);
    this.setValue = this.setValue.bind(this);
    this.columns = this.createPickerColumns();
    const initIndex = initPickerColumnsIndex.bind(this);
    this.pickerColumnsIndex = initIndex();
    this.setDate = this.setDate.bind(this);
    this.setInitializing = this.setInitializing.bind(this);
  }

  setValue(values) {
    const is12Hours = !this.config.is24Hours;
    let isPM = false;
    if (is12Hours) {
      const ampmIndex = Object.keys(this.pickerColumnsIndex).findIndex((col) => col === '12Hours');
      if (ampmIndex >= 0) {
        isPM = values[values.length - 1] === 'pm';
      }
    }
    Object.keys(this.pickerColumnsIndex).forEach((key, i) => {
      switch (key) {
        case 'YYYY':
          this.dateTime.setFullYear(values[i]);
          break;
        case 'MM':
          this.dateTime.setMonth(values[i]);
          break;
        case 'DD':
          this.dateTime.setDate(values[i]);
          break;
        case 'hh':
          {
            let hour = parseFloat(values[i]);
            if (isPM) {
              hour = hour < 12 ? hour + 12 : hour;
            } else {
              hour = hour === 12 ? 0 : hour;
            }
            this.dateTime.setHours(hour);
          }
          break;
        case 'mm':
          this.dateTime.setMinutes(values[i]);
          break;

        default:
          break;
      }
    });
  }

  setDate(date) {
    this.dateTime = date;
  }

  setInitializing(init) {
    this.initializing = init;
  }

  getValue() {
    return this.dateTime;
  }

  getPickerValues() {
    const values = [];
    const dateTime = this.dateTime;
    const year = dateTime.getFullYear();
    const month = dateTime.getMonth();
    const day = dateTime.getDate();
    let hour = dateTime.getHours();
    const minutes = dateTime.getMinutes();

    Object.keys(this.pickerColumnsIndex).forEach((key, i) => {
      switch (key) {
        case 'YYYY':
          values[i] = year;
          break;
        case 'MM':
          values[i] = month;
          break;
        case 'DD':
          values[i] = day;
          break;
        case 'hh': {
          if (hour >= 1 && hour <= 11) {
            values[i] = hour;
          } else if (hour === 0 || hour === 12) {
            values[i] = '12';
          } else {
            hour -= 12;
            values[i] = hour;
          }
          break;
        }
        case 'mm':
          values[i] = minutes;
          break;
        case '12Hours': {
          if (hour >= 0 && hour <= 11) {
            values[i] = 'am';
          } else {
            values[i] = 'pm';
          }
          break;
        }
        default:
          break;
      }
    });
    return values;
  }

  updateDateColumns() {
    const days = {
      values: [],
      displayValues: [],
      name: 'DD',
    };
    const monthlyDays = getMonthlyDay(this.dateTime.getFullYear(), this.dateTime.getMonth() + 1);
    for (let l = 1; l <= monthlyDays; l += 1) {
      days.values.push(l);
      days.displayValues.push(l);
    }
    this.columns = this.columns.map((col) => {
      if (col.name === 'DD') {
        return days;
      }
      return col;
    });
  }

  createPickerColumns() {
    const config = this.config;
    const dateFormatArr = config.pickerFormat.split(/-|\/|\s|:/g);

    const formatArr = dateFormatArr;

    const len = formatArr.length;
    const columns = [];
    const date = this.dateTime;
    for (let i = 0; i < len; i += 1) {
      const f = formatArr[i];

      if (f === 'YYYY') {
        const years = {
          values: [],
          displayValues: [],
          name: f,
          onChange: (picker, value) => {
            if (this.initializing) {
              return;
            }
            this.isUpdating = true;
            const index = this.pickerColumnsIndex.DD;
            const pickedYear = parseInt(value, 10);
            const maxDay = getMonthlyDay(pickedYear, this.dateTime.getMonth() + 1);
            const currentDay =
              picker.cols[index].value > maxDay ? maxDay : picker.cols[index].value;
            this.dateTime.setFullYear(value, this.dateTime.getMonth(), currentDay);
            this.updateDateColumns();
            if (picker.cols[index] && picker.cols[index].replaceValues) {
              picker.cols[index].replaceValues(
                this.columns[index].values,
                this.columns[index].displayValues,
              );
            }
            if (picker.cols[index] && picker.cols[index].setValue) {
              picker.cols[index].setValue(currentDay, 0);
            }
            this.isUpdating = false;
          },
        };
        for (let j = config.startYear; j <= config.endYear; j += 1) {
          years.values.push(j);
          years.displayValues.push(this.yearFormatter.format(new Date().setFullYear(j)));
        }
        columns.push(years);
      } else if (f === 'MM') {
        const months = {
          values: [],
          displayValues: [],
          name: f,
          onChange: (picker, value) => {
            if (this.initializing) {
              return;
            }
            this.isUpdating = true;
            const month = parseInt(value, 10);
            const index = this.pickerColumnsIndex.DD;
            const maxDay = getMonthlyDay(this.dateTime.getFullYear(), month + 1);
            const currentDay =
              picker.cols[index].value > maxDay ? maxDay : picker.cols[index].value;
            this.dateTime.setMonth(value, currentDay);
            this.updateDateColumns();

            if (picker.cols[index] && picker.cols[index].replaceValues) {
              picker.cols[index].replaceValues(
                this.columns[index].values,
                this.columns[index].displayValues,
              );
            }
            if (picker.cols[index] && picker.cols[index].setValue) {
              picker.cols[index].setValue(currentDay, 0);
            }

            this.isUpdating = false;
          },
        };
        for (let k = 0; k <= 11; k += 1) {
          months.values.push(k);
          const monthName = this.monthFormatter.format(new Date('0001-01-01').setMonth(k));
          months.displayValues.push(monthName);
        }
        columns.push(months);
      } else if (f === 'DD') {
        const days = { values: [], displayValues: [], name: f };
        const day = getMonthlyDay(date.getFullYear(), date.getMonth() + 1);
        for (let l = 1; l <= day; l += 1) {
          days.values.push(l);
          days.displayValues.push(this.dayFormatter.format(new Date('0001-01-01').setDate(l)));
        }
        columns.push(days);
      } else if (f === 'hh') {
        if (config.dateTimePicker) {
          columns.push({
            divider: true,
            content: '&nbsp;&nbsp;',
          });
        }
        const hours = {
          values: [],
          displayValues: [],
          name: f,
        };
        const maxHour = config.is24Hours ? 23 : 12;
        const minHour = config.is24Hours ? 0 : 1;
        for (let m = minHour; m <= maxHour; m += 1) {
          hours.values.push(m);
          hours.displayValues.push(m < 10 ? `0${m}` : m);
        }
        columns.push(hours);
      } else if (f === 'mm') {
        const minutes = {
          values: [],
          displayValues: [],
          name: f,
        };
        for (let n = 0; n <= 59; n += config.minStep) {
          minutes.values.push(n);
          minutes.displayValues.push(n < 10 ? `0${n}` : n);
        }
        columns.push(minutes);
      }
    }
    if (!config.is24Hours && !config.datePicker) {
      columns.push({
        name: '12Hours',
        values: ['am', 'pm'],
        displayValues: ['AM', 'PM'],
      });
    }
    return columns;
  }

  formatDate(d) {
    const calendar = this;
    const date = new Date(d);
    const year = date.getFullYear();
    const month = date.getMonth();
    const month1 = month + 1;
    const day = date.getDate();
    const { dateFormat, locale, dateTimePicker } = calendar.config;
    function twoDigits(number) {
      return number < 10 ? `0${number}` : number;
    }
    if (typeof dateFormat === 'string') {
      const tokens = {
        yyyy: year,
        yy: String(year).substring(2),
        mm: twoDigits(month1),
        m: month1,
        MM: this.longMonthFormatter.format(date),
        M: this.shortMonthFormatter.format(date),
        dd: twoDigits(day),
        d: day,
      };
      if (dateTimePicker) {
        const hours = date.getHours();
        const minutes = date.getMinutes();
        const seconds = date.getSeconds();
        let hours12 = hours;
        if (hours > 12) hours12 = hours - 12;
        if (hours === 0) hours12 = 12;
        const a = hours >= 12 && hours !== 0 ? 'pm' : 'am';

        Object.assign(tokens, {
          HH: twoDigits(hours),
          H: hours,
          hh: twoDigits(hours12),
          h: hours12,
          ss: twoDigits(seconds),
          s: seconds,
          ':mm': twoDigits(minutes),
          ':m': minutes,
          a,
          A: a.toUpperCase(),
        });
      }
      const regexp = new RegExp(
        Object.keys(tokens)
          .map((t) => `(${t})`)
          .join('|'),
        'g',
      );
      return dateFormat.replace(regexp, (token) => {
        if (token in tokens) return tokens[token];
        return token;
      });
    }

    if (typeof dateFormat === 'function') {
      return dateFormat(date);
    }

    // Intl Object
    const formatter = new Intl.DateTimeFormat(locale, dateFormat);
    return formatter.format(date);
  }
}
export default DateTime;
